/* $Workfile: b8tst.c $       $Revision: 22 $      $Modtime: 6/23/98 7:44p $  */
/*------------------------------------------------------- File Description -*\

  Odin TeleSystems Inc.

  NAME

    B8TST.C

  DESCRIPTION

    Test program to be used to test Balder-8S boards.

  PREPARATIONS

    Connect the Line Interfaces with the external Li loop cable
    (Li0<->Li1, L2<->Li3, ...).

\*--------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <conio.h>
#include <ctype.h>
#include <time.h>


#include "balder.h"              /* Balder Standard Application Programming Interface */

#include "logf.h"
#include "cmds.h"

#define DEFAULT_LOG_FILE "b8tst.log"

#define MAX_FRAME_LEN 115
//#define MAX_FRAME_LEN 20


#define NO_OF_BOARDS           1         /* We can have up to 4 boards         */
#define BALDER_BOARD_TYPE    128         /* Unique code for Balder-8 2.0       */
#define MAX_LI_NO              7
#define NO_OF_PIPES          (MAX_LI_NO + 1)
#define BCH1_ALSO              1


/*--------------------------------------------------------------------------*\
  Type Definitions
\*--------------------------------------------------------------------------*/

typedef unsigned char  Byte;
typedef unsigned int   Uint;
typedef unsigned short Word;
typedef unsigned long  Ulong;

#ifdef __cplusplus
  enum Bool { OTS_FALSE = 0, OTS_TRUE  = 1 } ;
#else
  typedef enum { OTS_FALSE = 0, OTS_TRUE  = 1 } Bool;
#endif


// Environment parameters:
typedef struct envT {
  short irq;
  short ioAddr;
  short liMode;
  BldLawType law;
  char *logFileName;
  Bool  performHandsetTest;
  Bool  performDtmfTest;
  Bool  performToneTest;
  Bool  performFskTest;
  Bool  performHdlcTest;
  Bool  exitPrompt;
  Ulong  frameCountNoDelay;
  Ulong  frameCountEnd;
  Ulong  dtmfTestCount;
  Ulong  fskTestCount;
} envT;


typedef enum {
#if 0
  BMT_NONE1,
  BMT_NONE2,
  BMT_NONE3,
#endif
  BMT_DTMF,
  BMT_CONST_BYTE,
  BMT_END
} BchannelGenT;

typedef struct {
  short        bchNo;
  Bool         active;
  BchannelGenT genType;
  short        genVal;
} BchannelMsgT;

// Random message test
typedef struct MsgT {
  Ulong seqNo;
  short len;
  Byte data[MAX_FRAME_LEN];

  BchannelMsgT bch1;
  BchannelMsgT bch2;

  long  sendTime;

  struct MsgT* next;
} MsgT;


typedef struct {
  Ulong constOkCount[NO_OF_PIPES];
  Ulong dtmfOkCount[NO_OF_PIPES];
  Ulong dtmfWaitCount[NO_OF_PIPES];
  Ulong dtmfErrCount[NO_OF_PIPES];
  Ulong rxFrameCount[NO_OF_PIPES];
  Ulong rxByteCount[NO_OF_PIPES];
  Ulong txFrameCount[NO_OF_PIPES];
  Ulong txByteCount[NO_OF_PIPES];
  Bool  active;
  Ulong busyCount[NO_OF_PIPES];
  short rxPipe[NO_OF_PIPES];  // Pipe that will receive this message
  short startPipe;
  short noOfPipes;
  short delayTime;

  Bool  bch1InUse[NO_OF_PIPES];
  Bool  bch2InUse[NO_OF_PIPES];
} RandSendT;


typedef struct {
  unsigned char data[256];
  short len;
  short bChannel;
} FskMsg;


/*--------------------------------------------------------------------------*\
  Global Data
\*--------------------------------------------------------------------------*/



/*--------------------------------------------------------------------------*\
  Private Data
\*--------------------------------------------------------------------------*/
static envT env;
static BldRc layer1Status[MAX_LI_NO + 1];    // Layer 1 status messages
MsgT* msgQueP[NO_OF_PIPES];  // One per pipe
MsgT* lastMsgP[NO_OF_PIPES]; // One per pipe
static RandSendT rs;
static FskMsg fskMsg;


/*--------------------------------------------------------------------------*\
  Function Prototypes
\*--------------------------------------------------------------------------*/

static void installDriver(short baseIoAddr, short irq);
static Bool idleFunction(void);
static void myExit(short exitCode);
static void parseCmdLineArgs(short argc, char *argv[], envT *env);
static void helpPanel(void);
static void exitQuestion(void);
static void randomSending(void);
static void myDelay(long milliseconds, Bool keyboardAttn);
static long myClock(void);  // returns milliseconds

static void performToneTest(
  short firstStimuliLi,
  short firstDetectLi
);


static void performDtmfTest(
  short firstStimuliLi,
  short firstDetectLi
);


static void performFskTest(
  char *fileName,
  short bChannel,
  short expectedMsgLen,
  unsigned char *expectedMsg
);


static void initBchannelMsg(
  short liNo,
  short bChannel,
  BchannelMsgT *bChMsgP,
  Bool *bchInUse
);


static void checkBchannelMsg(
  short liNo,
  short bChannel,
  BchannelMsgT *bChMsgP,
  MsgT *currMsgP,
  Bool *bchInUse
);



void main(short argc, char *argv[])
/*--------------------------------------------------------------------------*\
  Main Routine
\*--------------------------------------------------------------------------*/
{
  BldRc bldRc;
  short liNo;
  time_t startTime;

  printf("B8TST, %s, Odin TeleSystems Inc.\n", __DATE__);
  printf("Test program to be used to test Balder-8S boards.\n");


#if 0 // Test myClock and myDelay()
  {
    short n;
    printf("%ld \n", myClock());
    for (n = 0; n < 10; n++) {
      myDelay(1000L, OTS_TRUE);
      printf("%ld \n", myClock());
    }
    exit(0);
  }
#endif


  // Defaults:
  env.irq                = 10;
  env.ioAddr             = 0x280;
  env.logFileName        = DEFAULT_LOG_FILE;
  env.law                = LT_ALAW;

  env.performHandsetTest = OTS_FALSE;
  env.performDtmfTest    = OTS_FALSE;
  env.performToneTest    = OTS_FALSE;
  env.performHdlcTest    = OTS_FALSE;
  env.performFskTest     = OTS_FALSE;

  env.exitPrompt         = OTS_TRUE;
  env.frameCountNoDelay  = 300;  // Switch to 0 delay between frames when this many messages have been received
  env.frameCountEnd      = 5000; // End the test after this many received frames (on each pipe)
  env.dtmfTestCount      = 100;
  env.fskTestCount       = 1;

  parseCmdLineArgs(argc, argv, &env);

  if ((logF = fopen(env.logFileName, "wt")) == NULL) {
    printf("Couldn't open logfile %s\n", env.logFileName);
    exit(1);
  }

  installDriver(env.ioAddr, env.irq);

  for (liNo=0; liNo <= MAX_LI_NO; liNo++) {
    short attempt;
    // Activate line interfaces

    if (liNo % 2 == 1) continue;

    if (layer1Status[liNo] != BLD_L1_DOWN) {
      logf("Li#%d already activated. Skipped.\n", liNo);
      continue;
    }

    for (attempt = 1; attempt <= 1; attempt++) {

      logf("Activating Li#%d (%d of %d) \n", liNo, attempt, 5);

      if ((bldRc = bldActivate(liNo, 500L)) != BLD_SUCCESS) {
        logf("#%d: Error #%d: %s\n", liNo, (short) bldRc, bldGetErrMsg(bldRc));
      } else {
        break; // Activated...
      }
    }

    if (bldRc != BLD_SUCCESS) {
      logf("Max Activate attempts reached and the line is still not activated\n");
      exitQuestion();
    }

  }

  logf("Test starting...\n");


  /*----------------------------------------------------------------*\
    Start testing...
  \*----------------------------------------------------------------*/

  // Through-Connection Test
  {

    if (env.performHandsetTest) {
      Bool tctTestDone;
      short bChannelNo;
      BldRc rc;

      for (liNo = 0; liNo <= MAX_LI_NO; liNo += 2) {

        tctTestDone = OTS_FALSE;
        bChannelNo = 2;


        myDelay(160L, OTS_FALSE);

        logf("\nConnect handset to Li%d and Li%d\n", liNo, liNo+1);
        if ((rc = bldPhoneOn(liNo, bChannelNo, 0, AIS_MIC, AOS_RX, DG_MINUS_6DB, SG_MINUS_6DB)) != BLD_SUCCESS) {
          logf("Error: %s\n", bldGetErrMsg(rc));
          exitQuestion();
        }

        myDelay(160L, OTS_FALSE);

        if ((rc = bldPhoneOn((short)(liNo + 1), bChannelNo, 1, AIS_MIC, AOS_RX, DG_MINUS_6DB, SG_MINUS_6DB)) != BLD_SUCCESS) {
          logf("Error: %s\n", bldGetErrMsg(rc));
          exitQuestion();
        }

        logf("\nPlease verify Through-Connection between the two handsets\n");
        logf("\n");
        logf("Are the handsets through-connected?\n");
        logf("\n");
        logf("Press 'y' for Yes. Press 'n' for No.\n");

        while (!tctTestDone) {
          if (kbhit()) {
            switch (tolower(getch())) {
            case 'n':
              exitQuestion();
              break;
            case 'y':
              logf("TCT test passed.\n");
              tctTestDone = OTS_TRUE;
              break;
            default:
              logf("Press 'y' for Yes. Press 'n' for No.\n");
              break;
            }
          }
        }
        logf("\n");
      }

      logf("Reading status messages for 1 seconds...\n");
      startTime = time(NULL);
      while (time(NULL) <= startTime + 1) {
        idleFunction();  // Read from D-channels
      }

    } else {
      logf("Reading status messages for 1 seconds...\n");
      startTime = time(NULL);
      while (time(NULL) <= startTime + 1) {
        idleFunction();  // Read from D-channels
      }
    }
  }


  logf("Reading status messages for 1 seconds...\n");
  startTime = time(NULL);
  while (time(NULL) <= startTime + 1) {
    idleFunction();  // Read from D-channels
  }

  // DTMF and Silence test
  if (env.performDtmfTest) {
    performDtmfTest(0, 1);
    performDtmfTest(1, 0);
  }


  // Test using Sinewave instead of DTMF
  if (env.performToneTest) {
    performToneTest(0, 1);
    performToneTest(1, 0);
  }

  // Test FSK detection
  if (env.performFskTest) {
    short bChannel;
    Ulong count;

    for (count = 0; count < env.fskTestCount; count++) {

      logf("FSK Test No: %lu (of %lu)\n", count, env.dtmfTestCount);
      for (bChannel = 1; bChannel < 2; bChannel++) {
        logf("FSK Test on Bch%d\n", bChannel);
        performFskTest("bchan.rec", bChannel, 30, "\x80\x1B\x11\x01\x01\x01\x08\x31\x30\x31\x35\x31\x31\x35\x30\x02\x0C\x30\x31\x32\x37\x33\x2D\x35\x30\x30\x35\x35\x36\x4E");
        performFskTest("g150",      bChannel,  6, "\x80\x03\x31\x35\x30\xE7");
        performFskTest("g256",      bChannel,  6, "\x80\x03\x32\x35\x36\xE0");
        performFskTest("g7500",     bChannel,  7, "\x80\x04\x37\x35\x30\x30\xB0");
        performFskTest("g1234",     bChannel, 13, "\x80\x0A\x31\x32\x33\x34\x35\x36\x37\x38\x39\x30\x69");
      }
    }
  }




  // Setup for Random transmitting and receiving of random messages
  if (env.performHdlcTest) {
    short pipe;
    Bool switchedToNoDelay = OTS_FALSE;
    Bool allPipesDone;
    Bool pipeDone[NO_OF_PIPES];

    rs.active = OTS_TRUE;
    logf("Turning Random sending %s\n", rs.active ? "ON" : "OFF");

    memset(&rs, 0, sizeof(RandSendT));
    rs.active = OTS_TRUE;

    rs.delayTime = 200; // Max Delay Time (between every 10th message) (0=No Delay)
    rs.noOfPipes = NO_OF_PIPES;

    rs.startPipe = 0;

    for (pipe = 0; pipe < NO_OF_PIPES; pipe++) {
      if ((pipe % 2) == 0) { // Even pipe Number
        rs.rxPipe[pipe] = pipe+1;
      } else {
        rs.rxPipe[pipe] = pipe-1;
      }
    }

    for (pipe = rs.startPipe; pipe < rs.startPipe + rs.noOfPipes; pipe++) {
      // Create sentinel node for the msg queue
      if ((msgQueP[pipe] = (MsgT *) calloc(1, sizeof(MsgT))) == NULL) {
        logf("Error: Out of memory\n");
        myExit(1);
      }
      msgQueP[pipe]->next = NULL;
      lastMsgP[pipe] = msgQueP[pipe];
    }
    logf("Random sending is RUNNING with Max Delay Time = %d\n", rs.delayTime);
    logf("Start pipe for transmit is %d.\n", rs.startPipe);
    logf("Transmitting and receiving on %d pipes.\n", rs.noOfPipes);


    logf("Test is ended at %lu received frames (on each pipe).\n", env.frameCountEnd);
    logf("Test is executed with 0 delay between frames after at %lu received frames.\n", env.frameCountNoDelay);
    memset(pipeDone, '1', NO_OF_PIPES * sizeof(Bool));
    for (pipe = 0; pipe < NO_OF_PIPES; pipe++) {
      pipeDone[pipe]    = OTS_FALSE;
    }

    for (;;) {  // Loop exited with break or myExit()
      short i;
      short handsetLiNo = 0;
      short handSetBchannelNo = 1;
      short c;

      #if 0
      if (time(NULL) > (startTime + (time_t)env.frameCountEnd/20L)) {
        logf("Error: Timeout in Random sending. Exiting...\n");
        myExit(1);
      }
      #endif

      for (i = 0; i < 10; i++) {
        idleFunction();  // Read from D-channels
      }
      if (rs.active) {
        randomSending();
      }

      allPipesDone = OTS_TRUE;
      for (pipe = 0; pipe < NO_OF_PIPES; pipe++) {

        if (!switchedToNoDelay && rs.rxFrameCount[pipe] >= env.frameCountNoDelay) {
          switchedToNoDelay = OTS_TRUE;
          logf("Switching to 0 delay\n");
          rs.delayTime = 0; // Max Delay Time (between every 10th message) (0=No Delay)
        }

        if (rs.rxFrameCount[pipe] >= env.frameCountEnd) {
          pipeDone[pipe] = OTS_TRUE;
        } else {
          allPipesDone = OTS_FALSE;
        }

      }

      if (allPipesDone) {
        logf("All Pipes are done with Random Sending Test\n");
        break;
      }

      if (kbhit()) {
        c = getch();
        switch (c) {
        case 'q':
          exitQuestion();
          break;

        case 'b':
          logf("B-channel Counts:\n");
          for (pipe = 0; pipe < NO_OF_PIPES; pipe++) {
            logf("Pipe %d: ConstOk: %.9lu DtOk: %.9lu DtWait: %.9lu DtErr: %.9lu \n", pipe, rs.constOkCount[pipe], rs.dtmfOkCount[pipe], rs.dtmfWaitCount[pipe], rs.dtmfErrCount[pipe]);
          }
          break;
        case 'f':
          logf("Frame Counts:\n");
          for (pipe = 0; pipe < NO_OF_PIPES; pipe++) {
            logf("Pipe %d: TxF: %.9lu TxB:%.9lu  RxF: %.9lu RxB:%.9lu, Bus: %lu\n", pipe, rs.txFrameCount[pipe], rs.txByteCount[pipe], rs.rxFrameCount[pipe], rs.rxByteCount[pipe], rs.busyCount[pipe]);
          }
          logf("\n");
          break;


        case ']':
          logf("Li %d, Bch %d, AIC %d. Handset connected\n", handsetLiNo, handSetBchannelNo, 0);

          if ((bldRc = bldPhoneOn(handsetLiNo, handSetBchannelNo, 0, AIS_MIC, AOS_RX, DG_MINUS_6DB, SG_MINUS_6DB)) != BLD_SUCCESS) {
            logf("Error: %s\n", bldGetErrMsg(bldRc));
            exitQuestion();
          }
          break;

        case '[':
          logf("Li %d, Bch %d, AIC %d. Handset connected\n", handsetLiNo, handSetBchannelNo, 1);
          if ((bldRc = bldPhoneOn(handsetLiNo, handSetBchannelNo, 1, AIS_MIC, AOS_RX, DG_MINUS_6DB, SG_MINUS_6DB)) != BLD_SUCCESS) {
            logf("Error: %s\n", bldGetErrMsg(bldRc));
            exitQuestion();
          }
          break;

        case '{':
        case '}':
          logf("Li %d. Handset disconnected\n", handsetLiNo);
          if ((bldRc = bldPhoneOff(handsetLiNo)) != BLD_SUCCESS) {
            logf("Error: %s\n", bldGetErrMsg(bldRc));
            exitQuestion();
          }
          break;

        case '<':
          logf("Bch1 selected for handset operation\n");
          handSetBchannelNo = 1;
          break;

        case '>':
          logf("Bch2 selected for handset operation\n");
          handSetBchannelNo = 2;
          break;


        case '0':
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
          /* Select LI to act as target for subsequent commands */
          logf("Li %d: Selected\n", (int) c - '0');
          handsetLiNo = c - '0';
          break;

        default:
          logf("Press 'q' to exit. Press 'f' for frame count. Press 'b' for B-channel count\n");
          break;
        }
      }
    }
  }

  logf("Normal program termination...\n");

  myExit(0);
}


static void randomSending()
/*--------------------------------------------------------------------------*\

  DESCRIPTION:


\*--------------------------------------------------------------------------*/
{
  short  txPipe;
  short  len;
  short  loopTimes;
  short i;
  short j;
  BldRc bldRc;
  MsgT *tempP;
  MsgT *currMsgP;
  static short counter = 0;
  static Ulong sendSeqNo = 0;
  //BchannelMsgT bChMsgType;
#if 0
  static Bool printedMemLowWarning = OTS_FALSE;
#endif

  txPipe = (rand() % rs.noOfPipes) + rs.startPipe;

  loopTimes = rand() % 10;

  if (++counter % 10 == 0) {
    if (rs.delayTime) {
      short thisDelay = rand() % rs.delayTime;
      logf("Delaying %d ms\n", thisDelay);
      myDelay((long)thisDelay, OTS_FALSE);
    }
  }

  for (i = 0; i < loopTimes; i++) {

    len = (rand() % (MAX_FRAME_LEN - 1)) + 1;
    currMsgP = lastMsgP[rs.rxPipe[txPipe]];
    currMsgP->seqNo = sendSeqNo++;
    currMsgP->len = len;
    for (j = 0; j < currMsgP->len; j++) {
      #if 1
      currMsgP->data[j] = (Byte) rand();
      #else
      currMsgP->data[j] = (Byte) sendSeqNo;
      #endif
    }
#if 0 // Temporary test, corrupting message to generate some error message to see if we get them
    if (counter % 1000 == 0) {
      static Byte aBuf[130];
      for (j = 0; j < currMsgP->len; j++) {
        aBuf[j] = 0xAA;
      }
      bldRc = bldWriteLi(txPipe, aBuf, (short) currMsgP->len);
      if (bldRc == BLD_SUCCESS) {
        logf("Corrupting this message on pipe %d (len=%d)\n", txPipe, currMsgP->len);
      }
    } else {
      bldRc = bldWriteLi(txPipe, currMsgP->data, (short) currMsgP->len);
    }
#else
    bldRc = bldWriteLi(txPipe, currMsgP->data, (short) currMsgP->len);

#endif


    if (bldRc == BLD_SUCCESS) {
      rs.txFrameCount[txPipe]++;
      rs.txByteCount[txPipe] += currMsgP->len;

      initBchannelMsg(txPipe, 1, &currMsgP->bch1, &rs.bch1InUse[txPipe]);
      initBchannelMsg(txPipe, 2, &currMsgP->bch2, &rs.bch2InUse[txPipe]);

      currMsgP->sendTime = myClock();

      // The message was sent. Store it in the msg fifo
      // Make a new sentinel node (which will be put at the end of the queue)
      if ((tempP = (MsgT *) calloc(1, sizeof(MsgT))) == NULL) {
        logf("Error: Out of memory (sendSeqNo: %lu)\n", sendSeqNo);
        myExit(1);
      }
      // Put message into the message queue (leave the empty sentinel at the end)
      lastMsgP[rs.rxPipe[txPipe]] = tempP;
      currMsgP->next = tempP;

    } else if (bldRc == BLD_TX_BUSY) {
      rs.busyCount[txPipe]++;
    } else {
      logf("Error #%d: %s\n", (short) bldRc, bldGetErrMsg(bldRc));
      exitQuestion();
    }
  }
}



static void initBchannelMsg(
  short liNo,
  short bChannel,
  BchannelMsgT *msg,
  Bool *bchInUse
)
{
  BldRc rc;

  msg->bchNo = bChannel;

  if (*bchInUse) {
    // The B-channel is occupied
    msg->genType = BMT_END; // No B-channelmessage in with this HDLC frames
    return;
  }

  msg->genType = rand() % BMT_END;


  switch(msg->genType) {
  case BMT_DTMF:
    {
      static char digitBuf[2];

      msg->genVal = '0' + (rand() % 10);

      digitBuf[0] = (char) msg->genVal;
      digitBuf[1] = '\0';

      if ((rc = bldDtmfOnBch(liNo, bChannel, digitBuf, 0, 0, DG_0DB)) != BLD_SUCCESS) {
        logf("Li%d, Bch%d: : Error: %s\n", liNo, bChannel, bldGetErrMsg(rc));
        exitQuestion();
        msg->genType = BMT_END; // No B-channelmessage in with this HDLC frames
      } else {
        *bchInUse = OTS_TRUE;
      }
    }
    break;

  case BMT_CONST_BYTE:
    {
      msg->genVal = rand() % 256;

      if ((rc = bldByteOnBch(liNo, bChannel, (unsigned char) msg->genVal)) != BLD_SUCCESS) {
        logf("Li%d, Bch%d: : Error: %s\n", liNo, bChannel, bldGetErrMsg(rc));
        exitQuestion();
        msg->genType = BMT_END; // No B-channelmessage in with this HDLC frames
      } else {
        *bchInUse = OTS_TRUE;
      }

    }
    break;

  }

}


static Bool idleFunction()
/*--------------------------------------------------------------------------*\

  DESCRIPTION:

    Reads a message from the board, and prints it (if a message is
    available).

\*--------------------------------------------------------------------------*/
{
  unsigned char *p;
  unsigned short i;
  Bool rc;


  #define BUF_SIZE  270 // Must be at least 270 to accomodate all
                        // .. possible (legal) ISDN messages
  static unsigned char msgBuf[BUF_SIZE];
  BldFrameHeader fmHeader;


  if (bldRead(msgBuf, BUF_SIZE, &fmHeader) != BLD_NO_FRAMES) {

    rc = OTS_TRUE;

    // Random Sending
    if (rs.active && (fmHeader.fmType == BLD_FM_HDLC)) {
      if (fmHeader.liNo > 7) {
        logf("ERROR: Invalid pipe no for randomSending %d\n", fmHeader.liNo);
        exitQuestion();
      } else {
        MsgT *currMsgP;
        short pipe;

        pipe = fmHeader.liNo;

        // Pop from the message queue
        if (msgQueP[pipe]->next == NULL) { // Defensive
          short j;
          logf("Received unsent message on pipe %d\n", pipe);
          logf("\nReceived message: \n");
          for (j = 0; j < fmHeader.fmLength; j++) {
            logf("%.2X ", msgBuf[j]);
          }
          logf("\n");
          exitQuestion();
        } else {
          currMsgP = msgQueP[pipe];
          msgQueP[pipe] = currMsgP->next;

          if (currMsgP->len != fmHeader.fmLength) {
            short j;
            logf("ERROR: Mismatch in Length (on pipe %d). Expected %d (seqNo: %lu), but got %d \n", pipe, currMsgP->len, currMsgP->seqNo, fmHeader.fmLength);
            logf("Expected message: \n");
            for (j = 0; j < currMsgP->len; j++) {
              logf("%.2X ", currMsgP->data[j]);
            }
            logf("\nReceived message: \n");
            for (j = 0; j < fmHeader.fmLength; j++) {
              logf("%.2X ", msgBuf[j]);
            }
            logf("\n");
            exitQuestion();
          } else if (memcmp(currMsgP->data, msgBuf, currMsgP->len)) {
            short j;
            logf("ERROR: Mismatch in contents of message (on pipe %d). Expected: (seqNo: %lu)\n", pipe, currMsgP->seqNo);
            for (j = 0; j < currMsgP->len; j++) {
              logf("%.2X ", currMsgP->data[j]);
            }
            logf("\nBut got: \n");
            for (j = 0; j < currMsgP->len; j++) {
              logf("%.2X ", msgBuf[j]);
            }
            logf("\n");
            exitQuestion();
          } else {
            rs.rxFrameCount[pipe]++;
            rs.rxByteCount[pipe] += fmHeader.fmLength;

            checkBchannelMsg(pipe, 1, &currMsgP->bch1, currMsgP, &rs.bch1InUse[pipe]);
            checkBchannelMsg(pipe, 2, &currMsgP->bch2, currMsgP, &rs.bch2InUse[pipe]);

            if ((rs.rxFrameCount[pipe] % 500) == 0) {
              logf("Pipe %d: TxF: %.9lu TxB:%.9lu  RxF: %.9lu RxB:%.9lu, Bus: %lu\n", pipe, rs.txFrameCount[pipe], rs.txByteCount[pipe], rs.rxFrameCount[pipe], rs.rxByteCount[pipe], rs.busyCount[pipe]);
            }
          }

          free(currMsgP);
        }
      }

    } else {

      logf(">> Li:%d ", (short) fmHeader.liNo);
      logf("#%03d " , (short) fmHeader.fmSeqNo);
      logf("%02d:"  , (short) fmHeader.hour);
      logf("%02d:"  , (short) fmHeader.min);
      logf("%02d."  , (short) fmHeader.sec);
      logf("%02d [" , (short) fmHeader.csec);
      logf("%.2X] " , (short) fmHeader.fmStatus);

      p = msgBuf;

      switch (fmHeader.fmType) {
      case BLD_FM_I: // Only used on auto-mode
        logf("I-frame ");
        for (i = 0; i < fmHeader.fmLength; i++) {
          logf("%02X ", (short) *p++);
        }
        break;
      case BLD_FM_UI: // Only used in auto-mode
        logf("UI-frame ");
        for (i = 0; i < fmHeader.fmLength; i++) {
          logf("%02X ", (short) *p++);
        }
        break;
      case BLD_FM_HDLC:
        for (i = 0; i < fmHeader.fmLength; i++) {
          logf("%02X ", (short) *p++);
        }
        break;
      case BLD_FM_HDLC1:
        logf("BCH1 ");
        for (i = 0; i < fmHeader.fmLength; i++) {
          logf("%02X ", (short) *p++);
        }
        break;
      case BLD_FM_HDLC2:
        logf("BCH2 ");
        for (i = 0; i < fmHeader.fmLength; i++) {
          logf("%02X ", (short) *p++);
        }
        break;
      case BLD_FM_STR: // '\0' terminated info string
        logf("%s", msgBuf);
        break;
      case BLD_FM_STAT: // Status code
        #ifdef B8_LITE
        logf("STATUS [%d]", msgBuf[0]);
        #else
        logf("STATUS [%d] %s", msgBuf[0], bldStatus2Str(msgBuf[0]));
        #endif
        break;
      case BLD_FM_DTMF1: // DTMF digit string
        logf("DTMF BCH1 Digit %s", msgBuf);
        break;
      case BLD_FM_DTMF2: // DTMF digit string
        logf("DTMF BCH2 Digit %s", msgBuf);
        break;
      case BLD_FM_FSK1: // FSK octet Bch1
        logf("FSK BCH1 Octet %.2X", msgBuf[0]);
        fskMsg.bChannel = 1;
        fskMsg.data[fskMsg.len++] = msgBuf[0];
        break;
      case BLD_FM_FSK2: // FSK octet Bch2
        logf("FSK BCH2 Octet %.2X", msgBuf[0]);
        fskMsg.bChannel = 2;
        fskMsg.data[fskMsg.len++] = msgBuf[0];
        break;
      default:
        logf("Error: Invalid frame type");
        exitQuestion();
        break;
      }
      logf("\n");;

      if (rs.active) {
        logf("ERROR: Received unexpected message during Random HDLC test.\n");
        exitQuestion();
      }

    }
  } else {
    rc = OTS_FALSE;
  }

  {
    short liNo;
    // Don't print this stuff until we have encountered an empty read buffer

    /* Check layer 1 status, print any change (Although deactivation and  */
    /* Activations are endicated by STR messages returned by drvRead()    */
    for (liNo = 0; liNo <= 1; liNo++) {
      BldRc rc;
      rc = bldGetStatusLi(liNo);
      if (layer1Status[liNo] != rc) {
        logf("Li:%d Layer 1 status = %s\n", liNo, (rc == BLD_L1_DOWN) ? "Down" : "OK");
        layer1Status[liNo] = rc;

        if (rs.active) {
          logf("ERROR: Layer1 status change during Random HDLC test.\n");
          exitQuestion();
        }
      }
    }
  }

  return rc;
}




static void checkBchannelMsg(
  short liNo,
  short bChannel,
  BchannelMsgT *bChMsgP,
  MsgT *currMsgP,
  Bool *bchInUse
)
{
  BldRc rc;

  switch(bChMsgP->genType) {

  case BMT_DTMF:
    {
      short checkAttempts;
      char dtmfDigit;

      // Check when this DTMF tone was turned on. If we get this message too early then we need to wait for a while so that the tone can be detected properly
      while (myClock() < (currMsgP->sendTime + 120)) {
        rs.dtmfWaitCount[liNo]++;
        if (kbhit()) {
          getch();
          logf("Waiting for DTMF tone; Clock: %ld, sendTime: %ld\n", myClock(), currMsgP->sendTime);
          exitQuestion();
        }
      }

      // Now we can check for DTMF tone detection
      for (checkAttempts = 0; checkAttempts < 5; checkAttempts++) {

        // Check that we received the correct DTMF digit
        rc = bldGetStatusDtmf(liNo, bChannel, &dtmfDigit);
        if ((rc != BLD_TONE) && (rc != BLD_NO_TONE)) {
          logf("Li%d, Bch%d, SeqNo %lu: Error: DTMF Detection, %s\n", liNo, bChannel, currMsgP->seqNo, bldGetErrMsg(rc));
          exitQuestion();
        }

        if (rc == BLD_TONE) {
          break; // We found a tone; Break out and analyze it
        }

        if (rc == BLD_NO_TONE) {
          rs.dtmfErrCount[liNo]++;
        }

      }

      if (rc == BLD_TONE) {
        if (dtmfDigit == (char) bChMsgP->genVal) {
          rs.dtmfOkCount[liNo]++;
        } else {
          logf("Li%d, Bch%d, SeqNo %lu: Error: DTMF Detection. Detected wrong digit. Expected %c. Received %c\n", liNo, bChannel, currMsgP->seqNo, (char) bChMsgP->genVal, dtmfDigit);
          exitQuestion();
        }
      } else {
        logf("Li%d, Bch%d, SeqNo %lu: Error: DTMF Detection. Max attempts reached. Still no tone. Waiting for digit %c\n", liNo, bChannel, currMsgP->seqNo, (char) bChMsgP->genVal);
        exitQuestion();

      }

      *bchInUse = OTS_FALSE;
    }
    break;

  case BMT_CONST_BYTE:
    {
      unsigned char bchByte;

      // Check when this constant byte was turned on. If we get this message too early then we need to wait for a while so that we have time to read the new byte
      while (myClock() < (currMsgP->sendTime + 120)) {
        rs.dtmfWaitCount[liNo]++;
        if (kbhit()) {
          getch();
          logf("Waiting for Constant Byte; Clock: %ld, sendTime: %ld\n", myClock(), currMsgP->sendTime);
          exitQuestion();
        }
      }

      // Check that we received the correct constant byte
      if ((rc = bldByteReadBch(liNo, bChannel, &bchByte)) != BLD_SUCCESS) {
        logf("Li%d, Bch%d, SeqNo %lu : Error: %s\n", liNo, bChannel, currMsgP->seqNo, bldGetErrMsg(rc));
        exitQuestion();
      } else if (bchByte != (unsigned char) bChMsgP->genVal) {
        logf("Li%d, Bch%d, SeqNo %lu : Error: Wrong constant byte detected. Expected 0x%.2X. Received 0x%.2X\n", liNo, bChannel, currMsgP->seqNo, bChMsgP->genVal, bchByte);
        exitQuestion();
      } else {
        rs.constOkCount[liNo]++;
      }

      *bchInUse = OTS_FALSE;
    }
    break;

  }
}




static void myExit(short exitCode)
/*--------------------------------------------------------------------------*\
\*--------------------------------------------------------------------------*/
{
  BldRc rc;
  short liNo;

  for (liNo=0; liNo <= MAX_LI_NO; liNo++) {
    bldPhoneOff(liNo);
  }

  if ((rc = bldDestructDriver()) != BLD_SUCCESS) {
    logf("Error #%d: %s\n", (short) rc, bldGetErrMsg(rc));
  }

  exit(exitCode);
}


static void parseCmdLineArgs(short argc, char *argv[], envT *env) {
/*--------------------------------------------------------------------------*\

  DESCRIPTION

    Parse the programs command line arguments and calls upon
    requested actions.

    There are some optional switches that can be used following
    the driver name on the command line. Switches are are
    separated by spaces, and can appear in arbitrary order.
    Switches consist of a hyphen and a letter:

  SIDE EFFECT

    env : The environment variable is filled out

\*------------------------------------------------------------------------*/
  short  i;

  for (i = 1; i < argc; i++) {        // Scan command line arguments
    strupr(argv[i]);
    if (*argv[i] == '-' || *argv[i] == '/') {  // Is argument a legal switch?
      if (*(argv[i]+1) == 'A') {          // IO-address
        env->ioAddr = (short) strtol(argv[i]+2, NULL, 0);
      } else if (*(argv[i]+1) == 'I') {   // Irq
        env->irq = (short) strtol(argv[i]+2, NULL, 0);
      } else if (*(argv[i]+1) == 'L') {   // Log file
        env->logFileName = argv[i]+2;
      } else if (*(argv[i]+1) == 'C') {   // Companding Law
        if (*(argv[i]+2) == 'A') {   // A-law
          env->law = LT_ALAW;
        } else if (*(argv[i]+2) == 'U') {   // U-law
          env->law = LT_ULAW;
        } else {
          printf("Error: Invalid companding law command line switch %s\n", argv[i]);
          helpPanel();
          exit(1);
        }
      } else if (*(argv[i]+1) == 'Y') {   // No prompt for Exit in exitQuestion()
        env->exitPrompt = OTS_FALSE;
      } else if (*(argv[i]+1) == 'P') {   // TCT (Phone Through-Connection-Test)
        env->performHandsetTest = OTS_TRUE;
      } else if (*(argv[i]+1) == 'H') {   // HDLC test
        env->performHdlcTest = OTS_TRUE;
      } else if (*(argv[i]+1) == 'N') {   // Tone test
        env->performToneTest = OTS_TRUE;
      } else if (*(argv[i]+1) == 'F') {   // Fsk test
        env->performFskTest = OTS_TRUE;
      } else if (*(argv[i]+1) == 'M') {   // Dtmf test
        env->performDtmfTest = OTS_TRUE;
      } else if (*(argv[i]+1) == 'D') {   // Switch to 0 delay between frames after this many frames
        env->frameCountNoDelay = strtoul(argv[i]+2, NULL, 0);
      } else if (*(argv[i]+1) == 'X') {   // End the test after this many received frames
        env->frameCountEnd = strtoul(argv[i]+2, NULL, 0);
      } else if (*(argv[i]+1) == 'T') {   // No of times to run the DTMF test
        env->dtmfTestCount = strtoul(argv[i]+2, NULL, 0);
      } else if (*(argv[i]+1) == 'K') {   // No of times to run the FSK test
        env->fskTestCount =  strtoul(argv[i]+2, NULL, 0);

      } else {
        printf("Error: Unknown command line switch %s\n", argv[i]);
        helpPanel();
        exit(1);
      }
    } else {
      printf("Error: Unknown command line switch %s\n", argv[i]);
      helpPanel();
      exit(1);
    }
  }

}


static void helpPanel()
/*--------------------------------------------------------------------------*\
\*--------------------------------------------------------------------------*/
{
  printf("\nCommand line switches:\n");
  printf(" /A<address>  : IoBaseAddress\n");
  printf(" /I<irq>      : Irq\n");
  printf(" /L<fileName> : Log file name\n");
  printf(" /S<size>     : memSize in MBytes (2 or 8)\n");
  printf(" /P           : Perform Through Connection Test (Phone Test)\n");
  printf(" /H           : Perform HDLC test\n");
  printf(" /N           : Perform Tone test\n");
  printf(" /M           : Perform DTMF test\n");
  printf(" /F           : Perform FSK test\n");
  printf(" /Y           : No Prompt for Exit at Error\n");
  printf(" /D<frames>   : Switch to 0 delay between frames after this many frames\n");
  printf(" /X<frames>   : End the test after this many received frames\n");
  printf(" /T<count>    : No of times to run the Tone/DTMF/Silence test\n");
  printf(" /K<count>    : No of times to run the FSK test\n");
  printf(" /C<a|u>      : Companding law (A-law or U-law)\n");
}


static void exitQuestion()
/*--------------------------------------------------------------------------*\
\*--------------------------------------------------------------------------*/
{
  if (env.exitPrompt) {
    logf("Exit, Debug, or Continue? (E/D/C)\n");

    while (kbhit()) getch(); // Empty keyboard buffer

    switch (toupper(getch())) {
    case 'E':
      myExit(1);
      break;
    case 'D':
      logf("... Entering debug mode. Press '?' for help\n");
      cmdsFromKeyboard();
      break;
    default:
      logf("... Continuing ...\n");
      break;
    }

  } else {
    logf("Exiting...\n");
    myExit(1);
  }
}



void myDelay(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

     Halts execution for a specified time period.

\*---------------------------------------------------------------------------*/
  long milliseconds,
  Bool keyboardAttn
)
{
  clock_t startTime = clock();
  clock_t now;
  for (;;) {
    now = clock();
    if (now < startTime) {
      // We wrapped (after 50 days)
      return;
    }

    #ifdef __BORLANDC__
    if (now > startTime + ((milliseconds * CLK_TCK) / 1000) ) {
      return;
    }
    #else
    if (now > startTime + milliseconds ) {
      return;
    }
    #endif

    if (keyboardAttn) {
      if (kbhit()) {
        getch();
        exitQuestion();
      }
    }

  }
}


long myClock(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

     Returns a relative clock in milliseconds.

\*---------------------------------------------------------------------------*/
  void
)
{
  #ifdef __BORLANDC__
  return (clock() / CLK_TCK) * 1000;
  #else
  return clock();
  #endif
}



/*---------------------------------------------------- Function Description -*\
  DESCRIPTION
    Sets up the driver by initializing the software with the
    proper IRQs and such, and then all LI are configured.

  RETURNS
    OTS_TRUE   // No error(s)
    OTS_FALSE  // Error(s)
\*---------------------------------------------------------------------------*/
static void installDriver(short baseIoAddr, short irq)          // funcdef
{
  BldRc rc; // Return code from BLD functions
  short liNo;


  if ((rc = bldConstructDriver(BALDER_BOARD_TYPE, baseIoAddr, NO_OF_BOARDS, irq)) != BLD_SUCCESS) {
    logf("Error #%d: %s\n", (short) rc, bldGetErrMsg(rc));
    exitQuestion();
  }

  if ((rc = bldInitDriver()) != BLD_SUCCESS) {
    logf("Error #%d: %s\n", (short) rc, bldGetErrMsg(rc));
    exitQuestion();
  }


  for (liNo=0; liNo <= MAX_LI_NO; liNo++) {
    LiMode liMode;
    /* Configure a line interface.                         */
    /* for generation we always configure all LIs as TEs   */

    bldTermLi(liNo, OTS_TRUE); // Connect terminating resistor


    liMode = (liNo % 2 == 0) ? BLD_TE : BLD_LT_S;

    logf("Configuring Li#%d in %s mode\n", liNo, (liMode == BLD_TE) ? "TE" : "NT");

    if ((rc = bldConfigureLi(liNo, liMode)) != BLD_SUCCESS) {
      if (rc == BLD_L1_DOWN) {
        logf("Li:%d  Configured OK, but Layer 1 is inactive\n", liNo);
        layer1Status[liNo] = BLD_L1_DOWN; /* Stops idleFunction from
                                          .. printing it again            */
      } else {
        logf("Li:%d  Error #%d: %s\n", liNo, (short) rc, bldGetErrMsg(rc));
        exitQuestion();
      }
    }
  }


  // Configure DSPs
#if 0 // This is done in B8Boot now
  {
    short liNo;

    for (liNo = 0; liNo <= MAX_LI_NO; liNo++) {
      logf("Loading DSP Application to DSP#%d (%c-law)\n", liNo, env.law == LT_ALAW ? 'A' : 'U' );
      if ((rc = bldConfigureDsp(liNo, "..\\..\\dtmffsk.out", "_c_int00", env.law)) != BLD_SUCCESS) {
        logf("Error: %s\n", bldGetErrMsg(rc));
        exitQuestion();
      }
      //demoDelay(100);
    }
  }
#else
  {
    short liNo;

    for (liNo = 0; liNo <= MAX_LI_NO; liNo++) {
      logf("Reseting DSP#%d (%c-law) \n", liNo, env.law == LT_ALAW ? 'A' : 'U');
      if ((rc = bldRestartDsp(liNo, env.law)) != BLD_SUCCESS) {
        logf("Error: %s\n", bldGetErrMsg(rc));
        exitQuestion();
      }
    }
  }
#endif


  // Configure Detectors
  {
    short dspNo;
    short bchNo;

    for (dspNo = 0; dspNo <= MAX_LI_NO; dspNo++) {
      for (bchNo = 1; bchNo <= 2; bchNo++) {
        logf("Configuring DTMF detector for DSP#%d (Bch%d)\n", dspNo, bchNo);

        if ((rc = bldConfigureDtmfDetector(dspNo, bchNo, 1, 0)) != BLD_SUCCESS) {
          logf("Error: %s\n", bldGetErrMsg(rc));
          exitQuestion();
        }

        #if 1
        logf("Configuring Silence detector for DSP#%d (Bch%d)\n", dspNo, bchNo);

        if ((rc = bldConfigureSilenceDetector(dspNo, bchNo, 1, 1, 4000)) != BLD_SUCCESS) {
          logf("Error: %s\n", bldGetErrMsg(rc));
          exitQuestion();
        }
        #endif
      }
    }
  }

}


static void performDtmfTest(
  short firstStimuliLi,
  short firstDetectLi
)
{
  short liNo;
  BldRc rc;
  Ulong count;
  char dtmfDigit;


  //myDelay(500);

#if 1 // IOM
  if ((rc = bldPhoneOn(1, 2, 0, AIS_MIC, AOS_RX, DG_MINUS_6DB, SG_MINUS_6DB)) != BLD_SUCCESS) {
    logf("Error: %s\n", bldGetErrMsg(rc));
    exitQuestion();
  }
#endif

  for (count = 0; count < env.dtmfTestCount; count++) {

    logf("DTMF Test No: %lu (of %lu)\n", count, env.dtmfTestCount);

    // Start sending silence on both B-channels (from even Lis)
    logf("Start sending silence on both B-channels (from even Lis)\n");
    for (liNo = firstStimuliLi; liNo <= MAX_LI_NO; liNo += 2) {

      #ifdef BCH1_ALSO
      if ((rc = bldByteOffBch(liNo, 1)) != BLD_SUCCESS) {
        logf("Li%d, Bch%d: : Error: %s\n", liNo, 1, bldGetErrMsg(rc));
        exitQuestion();
      }
      #endif

      if ((rc = bldByteOffBch(liNo, 2)) != BLD_SUCCESS) {
        logf("Li%d, Bch%d: : Error: %s\n", liNo, 2, bldGetErrMsg(rc));
        exitQuestion();
      }

    }

    myDelay(60L, OTS_FALSE);


    logf("Detect silence on odd Li's (NT mode)\n");
    for (liNo = firstDetectLi; liNo <= MAX_LI_NO; liNo += 2) {

      // Check that the silence detector is reporting silence on both B-channels
      #ifdef BCH1_ALSO
      if ((rc = bldGetStatusSilence(liNo, 1)) != BLD_NO_TONE) {
        logf("Li%d, Bch%d: : Error: Silence Detection, %s\n", liNo, 1, bldGetErrMsg(rc));
        exitQuestion();
      }
      #endif

      // Check that the silence detector is reporting silence on both B-channels
      if ((rc = bldGetStatusSilence(liNo, 2)) != BLD_NO_TONE) {
        logf("Li%d, Bch%d: Error: Silence Detection, %s\n", liNo, 2, bldGetErrMsg(rc));
        exitQuestion();
      }

    }

    logf("Send from even Li's (TE mode) \n");
    for (liNo = firstStimuliLi; liNo <= MAX_LI_NO; liNo += 2) {

      #ifdef BCH1_ALSO
      if ((rc = bldByteOnBch(liNo, 1, 0xAA)) != BLD_SUCCESS) {
        logf("Li%d, Bch%d: : Error: %s\n", liNo, 1, bldGetErrMsg(rc));
        exitQuestion();
      }
      #endif

      if ((rc = bldDtmfOnBch(liNo, 2, "5", 0, 0, DG_0DB)) != BLD_SUCCESS) {
        logf("Li%d, Bch%d: : Error: %s\n", liNo, 2, bldGetErrMsg(rc));
        exitQuestion();
      }
    }

    myDelay(60L, OTS_FALSE);

    logf("Detect on odd Li's (NT mode)\n");
    for (liNo = firstDetectLi; liNo <= MAX_LI_NO; liNo += 2) {
      Byte bchByte;

      #ifdef BCH1_ALSO
      // Check that we receive the correct constant byte on Bch1
      if ((rc = bldByteReadBch(liNo, 1, &bchByte)) != BLD_SUCCESS) {
        logf("Li%d, Bch%d: : Error: %s\n", liNo, 1, bldGetErrMsg(rc));
        exitQuestion();
      } else if (bchByte != 0xAA) {
        logf("Li%d, Bch%d: : Error: Wrong constant byte detected. Expected 0xAA, but got 0x%X\n", liNo, 1, bchByte);
        exitQuestion();
      }
      #endif


      // Check that we receive the correct DTMF digit on Bch2
      if ((rc = bldGetStatusDtmf(liNo, 2, &dtmfDigit)) != BLD_TONE) {
        logf("Li%d, Bch%d: : Error: DTMF Detection, %s\n", liNo, 2, bldGetErrMsg(rc));
        exitQuestion();
      } else if (dtmfDigit != '5') {
        logf("Li%d, Bch%d: : Error: Wrong DTMF digit detected. Expected 5, but got %c\n", liNo, 2, dtmfDigit);
        exitQuestion();
      }

      // Check that the silence detector is reporting non-silence on Bch2
      if ((rc = bldGetStatusSilence(liNo, 2)) != BLD_TONE) {
        logf("Li%d, Bch%d: : Error: Silence Detection, %s\n", liNo, 2, bldGetErrMsg(rc));
        exitQuestion();
      }

    }



    logf("Start sending silence on both B-channels (from even Lis)\n");
    for (liNo = firstStimuliLi; liNo <= MAX_LI_NO; liNo += 2) {

      #ifdef BCH1_ALSO
      if ((rc = bldByteOffBch(liNo, 1)) != BLD_SUCCESS) {
        logf("Li%d, Bch%d: : Error: %s\n", liNo, 1, bldGetErrMsg(rc));
        exitQuestion();
      }
      #endif

      if ((rc = bldByteOffBch(liNo, 2)) != BLD_SUCCESS) {
        logf("Li%d, Bch%d: : Error: %s\n", liNo, 2, bldGetErrMsg(rc));
        exitQuestion();
      }

    }

    myDelay(80L, OTS_FALSE);


    logf("Detect silence on odd Li's (NT mode)\n");
    for (liNo = firstDetectLi; liNo <= MAX_LI_NO; liNo += 2) {

      // Check that the silence detector is reporting silence on both B-channels
      #ifdef BCH1_ALSO
      if ((rc = bldGetStatusSilence(liNo, 1)) != BLD_NO_TONE) {
        logf("Li%d, Bch%d: : Error: Silence Detection, %s\n", liNo, 1, bldGetErrMsg(rc));
        exitQuestion();
      }
      #endif

      // Check that the silence detector is reporting silence on both B-channels
      if ((rc = bldGetStatusSilence(liNo, 2)) != BLD_NO_TONE) {
        logf("Li%d, Bch%d: Error: Silence Detection, %s\n", liNo, 2, bldGetErrMsg(rc));
        exitQuestion();
      }

    }

    if (kbhit()) {
      getch();
      exitQuestion();
    }
  }

  logf("DTMF test completed...\n");

}


static void performToneTest(
  short firstStimuliLi,
  short firstDetectLi
)
{
  short liNo;
  BldRc rc;
  Ulong count;

  myDelay(200L, OTS_FALSE);

#if 1 // IOM
  if ((rc = bldPhoneOn(firstDetectLi, 2, 0, AIS_MIC, AOS_RX, DG_MINUS_6DB, SG_MINUS_6DB)) != BLD_SUCCESS) {
    logf("Error: %s\n", bldGetErrMsg(rc));
    exitQuestion();
  }
#endif

  for (count = 0; count < env.dtmfTestCount; count++) {

    logf("Tone Test No: %lu (of %lu)\n", count, env.dtmfTestCount);

    // Start sending silence on both B-channels (from even Lis)
    logf("Start sending silence on both B-channels (from even Lis)\n");
    for (liNo = firstStimuliLi; liNo <= MAX_LI_NO; liNo += 2) {

      #ifdef BCH1_ALSO
      if ((rc = bldByteOffBch(liNo, 1)) != BLD_SUCCESS) {
        logf("Li%d, Bch%d: : Error: %s\n", liNo, 1, bldGetErrMsg(rc));
        exitQuestion();
      }
      #endif

      if ((rc = bldByteOffBch(liNo, 2)) != BLD_SUCCESS) {
        logf("Li%d, Bch%d: : Error: %s\n", liNo, 2, bldGetErrMsg(rc));
        exitQuestion();
      }

    }

    myDelay(120L, OTS_FALSE);


    logf("Detect silence on odd Li's (NT mode)\n");
    for (liNo = firstDetectLi; liNo <= MAX_LI_NO; liNo += 2) {

      // Check that the silence detector is reporting silence on both B-channels
      #ifdef BCH1_ALSO
      if ((rc = bldGetStatusSilence(liNo, 1)) != BLD_NO_TONE) {
        logf("Li%d, Bch%d: : Error: Silence Detection, %s\n", liNo, 1, bldGetErrMsg(rc));
        exitQuestion();
      }
      #endif

      // Check that the silence detector is reporting silence on both B-channels
      if ((rc = bldGetStatusSilence(liNo, 2)) != BLD_NO_TONE) {
        logf("Li%d, Bch%d: Error: Silence Detection, %s\n", liNo, 2, bldGetErrMsg(rc));
        exitQuestion();
      }

    }

    logf("Send from even Li's (TE mode) \n");
    for (liNo = firstStimuliLi; liNo <= MAX_LI_NO; liNo += 2) {

      #ifdef BCH1_ALSO
      if ((rc = bldByteOnBch(liNo, 1, 0xAA)) != BLD_SUCCESS) {
        logf("Li%d, Bch%d: : Error: %s\n", liNo, 1, bldGetErrMsg(rc));
        exitQuestion();
      }
      #endif

      if ((rc = bldToneOnBch(liNo, 2, TT_SINEWAVE, DG_0DB)) != BLD_SUCCESS) {
        logf("Li%d, Bch%d: : Error: %s\n", liNo, 2, bldGetErrMsg(rc));
        exitQuestion();
      }
    }

    myDelay(120L, OTS_FALSE);

    logf("Detect on odd Li's (NT mode)\n");
    for (liNo = firstDetectLi; liNo <= MAX_LI_NO; liNo += 2) {
      Byte bchByte;

      #ifdef BCH1_ALSO
      // Check that we receive the correct constant byte on Bch1
      if ((rc = bldByteReadBch(liNo, 1, &bchByte)) != BLD_SUCCESS) {
        logf("Li%d, Bch%d: : Error: %s\n", liNo, 1, bldGetErrMsg(rc));
        exitQuestion();
      } else if (bchByte != 0xAA) {
        logf("Li%d, Bch%d: : Error: Wrong constant byte detected. Expected 0xAA, but got 0x%X\n", liNo, 1, bchByte);
        exitQuestion();
      }
      #endif



      // Check that the silence detector is reporting non-silence on Bch2
      if ((rc = bldGetStatusSilence(liNo, 2)) != BLD_TONE) {
        logf("Li%d, Bch%d: : Error: Silence Detection, %s\n", liNo, 2, bldGetErrMsg(rc));
        exitQuestion();
      }

    }



    logf("Start sending silence on both B-channels (from even Lis)\n");
    for (liNo = firstStimuliLi; liNo <= MAX_LI_NO; liNo += 2) {

      #ifdef BCH1_ALSO
      if ((rc = bldByteOffBch(liNo, 1)) != BLD_SUCCESS) {
        logf("Li%d, Bch%d: : Error: %s\n", liNo, 1, bldGetErrMsg(rc));
        exitQuestion();
      }
      #endif

      if ((rc = bldByteOffBch(liNo, 2)) != BLD_SUCCESS) {
        logf("Li%d, Bch%d: : Error: %s\n", liNo, 2, bldGetErrMsg(rc));
        exitQuestion();
      }

    }

    myDelay(120L, OTS_FALSE);


    logf("Detect silence on odd Li's (NT mode)\n");
    for (liNo = firstDetectLi; liNo <= MAX_LI_NO; liNo += 2) {

      // Check that the silence detector is reporting silence on both B-channels
      #ifdef BCH1_ALSO
      if ((rc = bldGetStatusSilence(liNo, 1)) != BLD_NO_TONE) {
        logf("Li%d, Bch%d: : Error: Silence Detection, %s\n", liNo, 1, bldGetErrMsg(rc));
        exitQuestion();
      }
      #endif

      // Check that the silence detector is reporting silence on both B-channels
      if ((rc = bldGetStatusSilence(liNo, 2)) != BLD_NO_TONE) {
        logf("Li%d, Bch%d: Error: Silence Detection, %s\n", liNo, 2, bldGetErrMsg(rc));
        exitQuestion();
      }

    }

    if (kbhit()) {
      getch();
      exitQuestion();
    }
  }

  logf("Tone test completed...\n");

}


static void performFskTest(
  char *fileName,
  short bChannel,
  short expectedMsgLen,
  unsigned char *expectedMsg
)
{
  short liNo;
  short j;
  BldRc rc;
  #define PL_SAMPLE_BUFSIZE (4*102)
  static short sampleBuf[PL_SAMPLE_BUFSIZE];
  FILE *sampleF;
  short readCount;
  short noOfSkipSamples;
  short compandData = 1;


  for (liNo = 0; liNo <= MAX_LI_NO; liNo++) {
    logf("Li %d: Playing B-Channel samples from file %s on b-channel %d\n", liNo, fileName, bChannel);

    if ((sampleF = fopen(fileName, "rb")) == NULL) {
      logf("Li %d: Error could not open file %s\n", liNo, fileName);
      exitQuestion();
      break;
    }

    // Skip a random number of samples from the beginning of the file
    noOfSkipSamples = rand() % 102;
    logf("Skipping %d samples\n", noOfSkipSamples);
    fread(sampleBuf, 2, noOfSkipSamples, sampleF);

    memset(&fskMsg, 0, sizeof(FskMsg));


    // Fill the buffer with data from the buffer for the first time
    readCount = fread(sampleBuf, 2, PL_SAMPLE_BUFSIZE, sampleF);
    if (readCount != PL_SAMPLE_BUFSIZE) {
      // We reached EOF on this read so we did not fill the entire sample buffer. So pad it with 0's
      memset(sampleBuf + readCount, 0, (PL_SAMPLE_BUFSIZE - readCount) * sizeof(short));
    }

    // Give the first buffer to the driver and tell him to start sending the data
    if ((rc = bldPlaybackBchInitiate(liNo, bChannel, sampleBuf, PL_SAMPLE_BUFSIZE, compandData, DG_0DB)) != BLD_SUCCESS) {
      logf("Li%d, Bch%d: : Error: %s\n", liNo, bChannel, bldGetErrMsg(rc));
      exitQuestion();
    }

    while (!feof(sampleF)) {

      // Fill the buffer again with data from the file
      readCount = fread(sampleBuf, 2, PL_SAMPLE_BUFSIZE, sampleF);
      if (readCount != PL_SAMPLE_BUFSIZE) {
        // We reached EOF on this read so we did not fill the entire sample buffer. So pad it with 0's
        memset(sampleBuf + readCount, 0, (PL_SAMPLE_BUFSIZE - readCount) * sizeof(short));
      }

      // Loop until the driver has accepted this new buffer
      while (bldPlaybackBch(liNo, bChannel, sampleBuf) != BLD_SUCCESS) {
        idleFunction();
        if (kbhit()) {
          getch();
          exitQuestion();
        }
      }
    }

    // Loop until the driver has sent the last buffer
    while (bldPlaybackBchTerminate(liNo, bChannel, 0) != BLD_SUCCESS) {
      idleFunction();
      if (kbhit()) {
        getch();
        exitQuestion();
      }
    }

    fclose(sampleF);

    logf("Done\n");

    while (idleFunction()) {
      if (kbhit()) {
        getch();
        exitQuestion();
      }
    }
    if (fskMsg.bChannel != bChannel) {
      logf("Error: Wrong B-Channel for FSK message. Expected %d. Got %d\n", bChannel, fskMsg.bChannel);
      exitQuestion();
    } else if (fskMsg.len != expectedMsgLen) {
      logf("Error: Mismatch in FSK message length. Expected %d. Got %d\n", expectedMsgLen, fskMsg.len);
      logf("Expected message: \n");
      for (j = 0; j < expectedMsgLen; j++) {
        logf("%.2X ", expectedMsg[j]);
      }
      logf("\nReceived message: \n");
      for (j = 0; j < fskMsg.len; j++) {
        logf("%.2X ", fskMsg.data[j]);
      }
      logf("\n");
      exitQuestion();
    } else if (memcmp(expectedMsg, &fskMsg.data[0], fskMsg.len)) {
      logf("Error: Mismatch in FSK message content.\n");
      logf("Expected message: \n");
      for (j = 0; j < expectedMsgLen; j++) {
        logf("%.2X ", expectedMsg[j]);
      }
      logf("\nReceived message: \n");
      for (j = 0; j < fskMsg.len; j++) {
        logf("%.2X ", fskMsg.data[j]);
      }
      logf("\n");
      exitQuestion();
    } else {
      logf("Received FSK message matched! \n");
    }

  }

  logf("FSK test completed...\n");

}






