/* $Workfile: OUT2IMG.C $       $Revision: 4 $      $Modtime: 3/26/98 6:56p $  */
/*--------------------------------------------------------------------------*\

  Copyright (C) Odin TeleSystems Inc., 1996


  DESCRIPTION

    Converts a DSP COFF file (*.out or *.obj) to a static array in a C
    header file.

\*--------------------------------------------------------------------------*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#include "balder.h"              /* Balder Standard Application Programming Interface */




#ifdef WIN32
  #define HUGE
#else
  #define HUGE huge
#endif


#define IMAGE_BUF_SIZE 30000U
static unsigned short HUGE imageBuf[IMAGE_BUF_SIZE];


void main(int argc, char *argv[])
/*---------------------------------------------------------------------------*\
  MAIN
\*---------------------------------------------------------------------------*/
{
  FILE *fout;
  long imageEntryAddr;              // DSP Program Entry address; output from bldCreateDspProgImage()
  unsigned short noOfImageElements; // No of image elements returned in imageBuf after calling bldCreateDspProgImage()
  char *entryPoint = "_c_int00";
  unsigned short i;
  BldRc bldRc;


  printf("OUT2IMG, %s, Odin TeleSystems Inc. 1998\n", __DATE__);
  printf("Converts a DSP COFF file to a Balder-8 DSP Image File\n\n");
  if (argc != 3) {
    printf("Usage:\n");
    printf("  OUT2IMG <CoffFile> <HeaderFile>\n");
    exit(1);
  }


  printf("Converting COFF File %s ...\n", argv[1]);

  if ((bldRc = bldCreateDspProgImage(argv[1], entryPoint, (unsigned short *)imageBuf, IMAGE_BUF_SIZE,  &imageEntryAddr, &noOfImageElements)) != BLD_SUCCESS) {
    printf("Error: %s\n", bldGetErrMsg(bldRc));
    exit(1);
  }


  if ((fout = fopen(argv[2], "wt")) == NULL) {
    printf("Error: Could open not open output file %s\n", argv[2]);
    exit(1);
  }


  fprintf(fout, "// Dsp Program Image File for Balder-8 2.0\n\n");
  fprintf(fout, "// COFF File   : %s\n", argv[1]);
  fprintf(fout, "// Entry Point : %s\n\n", entryPoint);

  fprintf(fout, "#define DSP_PROG_IMAGE_LEN %u\n", noOfImageElements);
  fprintf(fout, "#define DSP_PROG_ENTRY_ADDR 0x%lXL\n\n", imageEntryAddr);

#ifdef WIN32
  fprintf(fout, "unsigned short dspProgImage[DSP_PROG_IMAGE_LEN] = {");
#else
  fprintf(fout, "unsigned short huge dspProgImage[DSP_PROG_IMAGE_LEN] = {");
#endif

  for (i = 0; i < noOfImageElements; i += 2) {
    if (i % 8 == 0) {
      fprintf(fout, "\n");
    }

    fprintf(fout, "0x%.4X,0x%.4X,  ", imageBuf[i], imageBuf[i+1]);

  }

  fprintf(fout, "\n};\n\n");

  fclose(fout);

  printf("Conversion complete. See file %s\n", argv[2]);

  exit(0);
}





