/* $Workfile: BLDDEF.H $       $Revision: 12 $      $Modtime: 6/24/98 8:11p $  */
/*-------------------------------------------------------- File Description -*\

  Copyright (C) Odin TeleSystems Inc., 1994-1997

  NAME

    BLDDEF.H

  DESCRIPTION

    Balder API Macro and type definitions

  Copyright (C) Odin TeleSystems Inc., 1994-1997

  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

\*---------------------------------------------------------------------------*/
#ifndef BLDDEF_H
#define BLDDEF_H

#ifdef __cplusplus
  extern "C" {
#endif


/*---------------------------------------------------------------------------*\
  MACRO DEFINITIONS
\*---------------------------------------------------------------------------*/
#ifdef B8_LITE
#define BOARD_PER_PC  1            // Per unit (PC) (1 counting)
#else
#define BOARD_PER_PC  4            // Per unit (PC) (1 counting)
#endif

#define BLD_NO_IRQ   -1            // Run in polling mode


/* Frame End Codes for fmStatus in BldFrameHeader */
#define BLD_FEC_OK            0x00 /* Good message (no errors)               */
#define BLD_FEC_ABORTED       0x01 /* Set if the message was aborted by the
                                   .. remote side, i.e. a sequence of 7 1's
                                   .. was detected                           */
#define BLD_FEC_BAD_CRC       0x02 /* Set if CRC is bad                      */
#define BLD_FEC_OVERRUN       0x04 /* Set if at least one byte of the frame
                                   .. has been lost due to hardware fifo
                                   .. overflow (PC too slow)                 */

#define BLD_FEC_TRUNCATED     0x10 /* Set if the message is longer than the
                                   .. buffer supplied by the application
                                   .. program                                */
#define BLD_FEC_OVERFLOW      0x20 /* Set if the application is too slow
                                   .. reading messages from the driver
                                   .. (messages lost) or if the driver
                                   .. received a message that was longer than
                                   .. its internal buffers (message
                                   .. truncated)                             */


/*---------------------------------------------------------------------------*\
  TYPE DEFINITIONS
\*---------------------------------------------------------------------------*/

#if !defined(MSC51) && !defined(__BORLANDC__)
#pragma pack(push, bld_def_h, 2)
#endif


typedef enum {
/*---------------------------------------------------------------------------*\
  Return codes for all of the BldXxxxx functions.

  (Design Note: Do not update the number without updating
  bldErrorStr[])
\*---------------------------------------------------------------------------*/
  BLD_UNDEFINED                   =  0,  /* The driver does not know what to return
                                         .. for this request                          */
  BLD_SUCCESS                     =  1,  /* All OK, no errors.                        */
  BLD_INVALID_MODE                =  2,  /* Supplied Li mode is not allowed.          */
  BLD_INVALID_BOARD_TYPE          =  3,  /* Supplied Board type is not supported by
                                         .. by this driver.                           */
  BLD_INVALID_BOARD_NO            =  4,  /* Driver was supplied a Board number which
                                         .. is not allowed (should be 0 <= NO <= 3).  */
  BLD_INVALID_LI_NO               =  5,  /* Driver was supplied a Li number which
                                         .. is not allowed (should be 0 <= NO <= 8).  */
  BLD_INVALID_B_CHANNEL           =  6,  /* Driver was supplied an illegal  B channel
                                         .. number (should be 1 <= NO <= 2).          */
  BLD_INVALID_IRQ_NO              =  7,  /* Driver was supplied an IRQ number which
                                         .. is not allowed (Not supported by
                                         .. the board)                                */
  BLD_INVALID_ADDR                =  8,  /* Driver was supplied an I/O-address which
                                         .. is not allowed (Not supported by the
                                         .. Board)                                    */
  BLD_NOT_SETUP                   =  9,  /* Functions has been called without a proper
                                         .. configuration of the driver.              */
  BLD_NO_CLOCKS                   = 10,  /* Could not enable clocks for the ISAC
                                         .. functions.  (??? is this used)            */
  BLD_NO_FRAMES                   = 11,  /* No full frames received and ready for
                                         .. reading.                                  */
  BLD_TX_BUSY                     = 12,  /* Transmitter not ready. Transmission of the
                                         .. previous frame has not been completed.    */
  BLD_TIMEOUT                     = 13,  /* Function timed out before the expected
                                         .. response was obtained from ISR or ISAC    */
  BLD_L1_OK                       = 14,  /* Physical Layer (L1) is up.                */
  BLD_L1_DOWN                     = 15,  /* Physical Layer (L1) is down,
                                         .. no connection.                            */
  BLD_BAD_CHIP                    = 16,  /* Line Interface (ISAC-S chip) did not
                                         .. respond correctly to a command            */
  BLD_NO_BOARD                    = 17,  /* No board, or IO base address mismatch
                                         .. between board and config file             */
  BLD_RESET_FAILED                = 18,  /* Driver could not find a board. I/O-address
                                         .. or IRQ mismatch between board and values
                                         .. supplied to the driver.                   */
  BLD_WRONG_CONTEXT               = 21,  /* Requested action is not valid in current
                                         .. context. Maybe the function call sequence
                                         .. was wrong                                 */
  BLD_ALREADY_ACTIVATED           = 22,  /* Activate request was issued but L1 was
                                         .. already activated                         */
  BLD_DSP_NOT_CONFIGURED          = 23,  /* DSP has not yet been configured           */
  BLD_DL_REL_IND                  = 24,  /* Layer-2 DM message received               */
  BLD_MDL_ERR_IND                 = 25,  /* Network does not respond                  */
  BLD_MDL_ERR_RESP                = 26,  /* Wrong response from network               */
  BLD_AUTO_MODE                   = 27,  /* Operating in Auto mode                    */
  BLD_NON_AUTO_MODE               = 28,  /* NOT operating in Auto mode                */
  BLD_INVALID_KEY                 = 29,  /* Invalid keypad key                        */
  BLD_BYTE_GEN_OCCUPIED           = 30,  /* B-channel byte generator occupied. Use    ..
                                            .. value 0                                   */
  BLD_INVALID_BYTE_VAL            = 31,  /* Byte value must be less than 128          */
  BLD_DSP_LOAD_FAILED             = 32,  /* Could not load DSP Application            */
  BLD_UNDER_CONSTRUCTION          = 33,  /* This function is under construction       */
  BLD_INVALID_AIC_NO              = 34,  /* AIC Number must be 0 or 1                 */
  BLD_DTMF_BUSY                   = 35,  /* Still sending previous set of DTMF numbers*/
  BLD_DTMF_NO_TOO_LONG            = 36,  /* DTMF phone number too long                */
  BLD_INVALID_DTMF_VAL            = 37,  /* Invalid DTMF value                        */
  BLD_DSP_COMM_FAILURE            = 38,  /* DSP<->Host Communication Failure          */
  BLD_DSP_NO_KERNEL_RESP          = 39,  /* No response from DSP Kernel               */
  BLD_FOPEN_ERROR                 = 40,  /* Could not open file                       */
  BLD_DSP_INVALID_ENTRYPT         = 41,  /* DSP Application Entry Point could not be
                                            .. located                                   */
  BLD_TONE                        = 42,  /* Tone is detected                          */
  BLD_NO_TONE                     = 43,  /* Tone is not detected                      */
  BLD_DSP_ADDR_MISMATCH           = 44,  /* HPI Address Mismatch. Internal Error      */
  BLD_DRV_CALL_FAILED             = 45,  /* Call to the Driver failed.                */
  BLD_DRV_NOT_INITIALIZED         = 46,  /* The Driver has not been initialized       */
  BLD_DATA_TOO_LARGE              = 47,  /* Data does not fit to the allocated buffer */
  BLD_MSG_TOO_LONG                = 48,  /* The supplied message is longer than
                                            .. internal buffers                          */
  BLD_DEVICE_OPEN_FAILED          = 49,  /* Unable to connect to the driver           */
  BLD_INVALID_CALLBACK_FUNCTION   = 50,  /* Provided callback function is not valid  */
  BLD_CALLBACK_ALREADY_SET        = 51,  /* Callback already set                     */
  BLD_CANT_CREATE_CALLBACK_THREAD = 52,  /* Unable to create a callback thread */
  BLD_INVALID_SAMPLE_BUF_SIZE     = 53,  /* sampleBufSize must a multiple of 102 */
  BLD_RECORD_BUF_EMPTY            = 54,  /* Recording buffer is empty */
  BLD_RECORD_BUF_HALF_FULL        = 55,  /* Recording buffer is half full */
  BLD_RECORD_BUF_FULL             = 56,  /* Recording buffer is completely full */
  BLD_OUT_OF_MEMORY               = 57,  /* Not enough memory to perform this operation */
  BLD_PLAYBACK_NOT_INITIATED      = 58,  /* Playback is not initiated */
  BLD_REC_BUSY                    = 59,  /* Recording busy */
  BLD_RECORD_NOT_INITIATED        = 60,  /* Recording is not initiated */


} BldRc;



/*---------------------------------------------------------------------------*\
  Status codes (can be converted to strings with call to bldStatus2Str())
\*---------------------------------------------------------------------------*/
typedef enum {
  LD_RETRANSMISSION_REQUEST              =  0,
  LD_TRANSMIT_DATA_UNDERRUN              =  1,
  LD_RECEIVE_FRAME_OVERFLOW              =  2,
  LD_ST_OVERFLOW                         =  3,
  LD_MONITOR_CH_STAT_CHANGE              =  4,
  LD_SUBSCRIBER_AWAKE                    =  5,
  LD_WATCHDOG_TIMER_OVERFLOW             =  6,
  LD_F3_DEACTIVATED_PU                   =  7,
  LD_F4_AWAITING_SIGNAL                  =  8,
  LD_F3_PENDING_DEACTIVATION             =  9,
  LD_F5_IDENTIFYING_INPUT                = 10,
  LD_F8_LOST_FRAMING                     = 11,
  LD_F6_SYNCHRONIZED                     = 12,
  LD_F7_ACTIVATED_8_9                    = 13,
  LD_F7_ACTIVATED_10_11                  = 14,
  LD_F3_DEACTIVATED_PD                   = 15,
  LD_F3_POWERUP_NO_TIM                   = 16,
  LD_SLIP_DETECTED                       = 17,
  LD_TE_DISCONNECTED                     = 18,
  LD_TE_RESET_STATE                      = 19,
  LD_TE_TEST_INDICATION                  = 20,
  LD_TE_AWAKE_TEST_INDICATION            = 21,
  LD_LI_MODE_NOT_SET                     = 22,
  LD_UNKNOWN_INDICATION                  = 23,
  LD_G4_PENDING_DEACTIVATION             = 24,
  LD_G3_ACTIVE_LSL                       = 25,
  LD_G2_PENDING_ACTIVATION               = 26,
  LD_G3_ACTIVE_SYNCHRONOUS               = 27,
  LD_G1_DEACTIVE                         = 28,
  LD_UNKNOWN_INDICATION_LT_S             = 29,
  LD_PROTOCOL_ERROR                      = 30,
  LD_DSP_FSK_ALERT_BCH1                  = 31,
  LD_DSP_FSK_ALERT_BCH2                  = 32,
  LD_DSP_RX_OVERFLOW                     = 33,
  LD_DSP_IOM_RESYNC                      = 34,
  LD_DSP_INVALID_H2D_CMD                 = 35,
  LD_DSP_CMD_BUF_OVERRUN                 = 36,
  LD_DSP_FSK_TIMEOUT_BCH1                = 37,
  LD_DSP_FSK_TIMEOUT_BCH2                = 38,
  LD_DSP_INTERNAL_ERROR                  = 39,
  LD_DSP_UNKNOWN_D2H_CMD                 = 40,
  LD_DSP_FSK_COMPLETE_BCH1               = 41,
  LD_DSP_FSK_COMPLETE_BCH2               = 42,
  LD_DSP_END_PLAYBACK_BCH1               = 43,
  LD_DSP_END_PLAYBACK_BCH2               = 44,
  LD_DSP_DTMF_COMPLETE_BCH1              = 45,
  LD_DSP_DTMF_COMPLETE_BCH2              = 46,
  LD_DSP_HDLC_BUF_OVERRUN_BCH1           = 47,
  LD_DSP_HDLC_BUF_OVERRUN_BCH2           = 48,
  LD_DSP_HDLC_PROTOCOL_ERR_BCH1          = 49,
  LD_DSP_HDLC_PROTOCOL_ERR_BCH2          = 50,
  LD_DSP_RECORD_OVERRUN_BCH1             = 51,
  LD_DSP_RECORD_OVERRUN_BCH2             = 52,
  LD_DRV_FIFO_OVERFLOW                   = 53,
  LD_DSP_H2D_ISR_CMD_OVERRUN             = 54,

  LD_MAX
} BldStatusCodeType;


typedef enum {
/*---------------------------------------------------------------------------*\
  Type of the receive/transmit frame (message).
\*---------------------------------------------------------------------------*/
  BLD_FM_HDLC,    /* HDLC frame on the D-channel (including Layer 2)         */
  BLD_FM_I,       /* I-frame (Only Layer 3, No Layer 2)                      */
  BLD_FM_UI,      /* UI-frame (Only Layer 3, No Layer 2)                     */
  BLD_FM_STR,     /* '\0' terminated string                                  */
  BLD_FM_STAT,    /* Status code from a device on the Board                  */
  BLD_FM_DTMF1,   /* DTMF digit string dectected on B-channel 1              */
  BLD_FM_DTMF2,   /* DTMF digit string dectected on B-channel 2              */
  BLD_FM_FSK1,    /* FSK octet dectected on B-channel 1                      */
  BLD_FM_FSK2,    /* FSK octet dectected on B-channel 2                      */
  BLD_FM_HDLC1,   /* HDLC frame received on B-channel 1                      */
  BLD_FM_HDLC2,   /* HDLC frame received on B-channel 2                      */
  BLD_FM_UNDEF
} BldFrameType;


typedef enum {
/*---------------------------------------------------------------------------*\
  Analog Input Source
\*---------------------------------------------------------------------------*/
  AIS_DISCONNECT, /* Don't use the AIC to drive the B-channel                */
  AIS_MIC,        /* Use the Handset Microphone as the input source to drive the B-channel      */
  AIS_AUX         /* Use the Aux input on the harmonica as the input source to drive the B-channel      */
} BldAnalogInSrc;

typedef enum {
/*---------------------------------------------------------------------------*\
  Analog Output Source
\*---------------------------------------------------------------------------*/
  AOS_DISCONNECT, /* Don't connect the speaker at all                        */
  AOS_RX,         /* Send received S-Bus samples to the speaker              */
  AOS_TX,         /* Send generated samples to the speaker                   */
  AOS_RX_AND_TX   /* Send both recieved S-bus samples and generated samples to the speaker */
} BldAnalogOutSrc;

typedef enum {
/*---------------------------------------------------------------------------*\
  Companding algorithms to be used for B-channel data
\*---------------------------------------------------------------------------*/
  LT_ULAW,      /* 8-bit u255 PCM code (American standard)                   */
  LT_ALAW       /* 8-bit A-law PCM code (European standard)                  */
} BldLawType;

typedef enum {
/*---------------------------------------------------------------------------*\
  Type of tone to be generated, see bldToneOnBch()
\*---------------------------------------------------------------------------*/
  TT_SINEWAVE,      /* 1kHz sinusoidal wave                                  */
  TT_SAWTOOTH,      /* Ramp wave                                             */
  TT_SWEEPING_SINE, /* Variable frequency sinusoidal wave                    */
  TT_NOISE,         /* Random noise                                          */
  TT_BYTE_VARIABLE  /* Sequential bytes: 0, 1, 2, ..., 255, 0, 1, ...        */
} BldToneType;

typedef enum {
/*---------------------------------------------------------------------------*\
  Digital gain of generated tones
\*---------------------------------------------------------------------------*/
  DG_0DB = 0,         /* Full Gain                                           */
  DG_MINUS_3DB = 1,   /* -3dB Gain                                           */
  DG_MINUS_6DB = 2,   /* -6dB Gain                                           */
  DG_MINUS_9DB = 3,   /* -9dB Gain                                           */
  DG_MINUS_12DB = 4,  /* -12dB Gain                                          */
  DG_MINUS_15DB = 5   /* -15dB Gain                                          */
} BldDigGainType;

typedef enum {
/*---------------------------------------------------------------------------*\
  Speaker gain
\*---------------------------------------------------------------------------*/
  SG_MINUS_6DB = 0,   /* -6dB Gain                                           */
  SG_MINUS_3DB = 1,   /* -3dB Gain                                           */
  SG_0DB = 2,         /* No Gain                                             */
  SG_PLUS_3DB = 3,    /* +3dB Gain                                           */
  SG_PLUS_6DB = 4     /* +6dB Gain                                           */
} BldSpkrGainType;


typedef enum {
/*---------------------------------------------------------------------------*\
  Operating modes for the Line Interface (Li).
\*---------------------------------------------------------------------------*/
  BLD_TE,         /* The line interface is configured to operate in Terminal
                  .. mode (as a TE).                                         */
  BLD_LT_S,       /* The line interface is configured to operate in Line
                  .. Termination mode (as an NT).                            */
  BLD_MON,        /* Used internally by bldConfigureMonitor(). Not to be used with bldConfigureLi() */
  BLD_NO_IMODE
} LiMode;


typedef enum {
/*---------------------------------------------------------------------------*\
  Frame Fill Type
\*---------------------------------------------------------------------------*/
  BLD_FFT_FLAGS    = 0x00,   // Send HDLC flags (0x7E) between the frames.
  BLD_FFT_ALL_ONES = 0x01    // Send ones between frames.
} BldFrameFillType;


typedef struct {
/*---------------------------------------------------------------------------*\
  Each received message is supplied with a data structure containing
  information on the received frame.
\*---------------------------------------------------------------------------*/
  short liNo;                /* Number of the Line Interface/DSP which received
                             .. the message.                                 */
  unsigned char  fmSeqNo;    /* Sequence number. All the received messages
                             .. are assigned a sequence number by the driver.
                             .. After 256 frames the numbering is started
                             .. again from 0.                                */
  unsigned short fmLength;   /* Length of the received message.              */
  unsigned char  hour;       /* Time of reception: Hour                      */
  unsigned char  min;        /* Time of reception: Minute                    */
  unsigned char  sec;        /* Time of reception: Second                    */
  unsigned char  csec;       /* Time of reception: Hundreds of second        */
  BldFrameType   fmType;     /* Type of the received frame                   */
  unsigned char  fmStatus;   /* Status of the received frame                 */
} BldFrameHeader;


#if !defined(MSC51) && !defined(__BORLANDC__)
#pragma pack(pop, bld_def_h, 2)
#endif


#ifdef __cplusplus
  } // extern "C"
#endif


/* $History: BLDDEF.H $
 * 
 * *****************  Version 12  *****************
 * User: Hanz         Date: 6/24/98    Time: 8:11p
 * Updated in $/Products/Bld8-20/Design/Drv/W95/VxD/Inc
 *
 * *****************  Version 11  *****************
 * User: Hanz         Date: 5/09/98    Time: 12:04p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Inc
 * Added bldConfigureMonitor() to configure a pair of Lis in monitor mode.
 * One Li will be configured in BLD_TE mode and the other in BLD_MON mode.
 * BLD_MON mode is basically BLD_TE mode, except that he does not know
 * about the line status. He will get the line status from his liPartner
 * (the BLD_TE guy).
 *
 * *****************  Version 10  *****************
 * User: Hanz         Date: 4/28/98    Time: 8:59p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Inc
 * Now we correctly initialize the Buffered Serial Port in the DSP
 * (connected to ISAC). We also reinitilalize this port when the the SBUS
 * status goes to L1S_OK. We also change the blink frequency depending on
 * if the SBUS status is L1S_OK or L1S_DOWN.
 *
 * *****************  Version 9  *****************
 * User: Hanz         Date: 4/03/98    Time: 12:59p
 * Updated in $/Products/Bld8-20/Design/Drv/W95/VxD/Inc
 * Just changed some comments.
 *
 * *****************  Version 8  *****************
 * User: Hanz         Date: 4/01/98    Time: 3:55p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Inc
 * Fixed a bug in playback and record where we did not switch between ping
 * and pong correctly. We did not give the host a decent chance to provide
 * the data to the driver before he needed it. Also changed the DSP->HOST
 * communcation. Now both the command and a 16-bit data is bufferered in
 * the command buffer. The buffer size is increased. The HDLC buffer size
 * on B-channels is increased to 128 bytes. Also the HDLC on Bch -> host
 * buffer is now a circular buffer (instead of a ping-pong buffer). This
 * gives the host ISR more latency before he needs to read the HDLC data
 * from the DSP.
 *
 * *****************  Version 7  *****************
 * User: Hanz         Date: 3/18/98    Time: 12:18p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Inc
 * Inplemented the recording functions with Ping Pong buffers (in the same
 * way as playback).
 *
 * *****************  Version 6  *****************
 * User: Hanz         Date: 3/18/98    Time: 7:42a
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Inc
 * Changed the plackback function so that he uses a ping-pong buffer for
 * data transferred from the application to the driver. This works in the
 * BC driver. B8TST runs OK.
 *
 * *****************  Version 5  *****************
 * User: Hanz         Date: 3/16/98    Time: 5:33p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Inc
 * Added support for Aborted HDLC frames on the B-channel. Also added the
 * FCS check. B8TST runs OK. HDLC B-channel frames only tested with a few
 * samples files (3ff, abort, and 3msg).
 *
 * *****************  Version 4  *****************
 * User: Hanz         Date: 3/15/98    Time: 10:44p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Inc
 * Added support for HDLC reception on the B-channels. Not quite
 * complemeted. Need to add two more HDLC fifos in the host driver, and
 * add  the FCS check.
 *
 * *****************  Version 3  *****************
 * User: Hanz         Date: 3/08/98    Time: 8:48p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Inc
 * Changed parameters to bldPhoneOn() so that the microphone and speaker
 * are treated as two separate entities. Started to implement a handset
 * "monitor" functionality where the handset speaker gets both the DSP
 * generated data and the received data from ISAC. This functionality does
 * not work yet. Also implemented separate speaker gain for the handset
 * speaker. Also implemented a "raw" playback/recording option.
 *
 * *****************  Version 2  *****************
 * User: Hanz         Date: 2/03/98    Time: 11:47p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Msc51/Inc
 * Corrected a bug where bldDtmfOnBch() would always report BUSY if he was
 * interrupted with a bldToneOffBch() call (for instance). Also added a
 * status message when the Dtmf sender has completed a phone number.
 *
 * *****************  Version 1  *****************
 * User: Hanz         Date: 1/25/98    Time: 10:13p
 * Created in $/Products/Bld8-20/Design/Drv/Dos/Msc51/Inc
 *
 * *****************  Version 18  *****************
 * User: Hanz         Date: 1/24/98    Time: 11:25p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Msc51
 * Implemented bldRecordBch(). Now works for both B-channels.
 *
 * *****************  Version 17  *****************
 * User: Hanz         Date: 1/24/98    Time: 8:23p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Msc51
 * Implemented a new version of playback where the ISR is sending the data
 * from a buffer supplied by the application.
 *
 * *****************  Version 16  *****************
 * User: Hanz         Date: 1/23/98    Time: 9:01p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Msc51
 * Fixed compiler warnings in MSC51. Added bldStatus2Str() function. Added
 * new fmType BLD_FM_STAT.
 *
 * *****************  Version 15  *****************
 * User: Hanz         Date: 1/23/98    Time: 2:58p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Msc51
 * If B8_LITE is defined we can only run 1 Board (this will make the
 * driver about 11k smaller).
 *
 * *****************  Version 14  *****************
 * User: Neil         Date: 12/17/97   Time: 11:32p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc
 * Added TT_BYTE_VARIABLE
 *
 * *****************  Version 13  *****************
 * User: Jari         Date: 12/09/97   Time: 1:44p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc Sohl Fix
 * Added BLD_NO_IRQ macro
 *
 * *****************  Version 12  *****************
 * User: Neil         Date: 11/19/97   Time: 6:59p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc
 * Debugging FSK detection. Circular buffer is receiving octets with no
 * value and DSP notifies host. However, the timeout counter does not
 * work, and the program normally cannot finish the WAIT_FOR_MARK loop.
 *
 * *****************  Version 11  *****************
 * User: Jari         Date: 11/11/97   Time: 12:46p
 * Updated in $/Products/Bld8-20/Design/Drv/W95/VxD
 * Added BldRc message strings
 *
 * *****************  Version 10  *****************
 * User: Lme          Date: 11/05/97   Time: 2:47p
 * Updated in $/Products/Bld8-20/Design/Drv/NT/Kmd
 * Implemented and alpha tested Event Notification.
 *
 * *****************  Version 9  *****************
 * User: Jari         Date: 10/23/97   Time: 4:41p
 * Updated in $/Products/Bld8-20/Design/Drv/W95/VxD
 * Converted int's to short's to make porting between 16 and 32-bit
 * platforms easier.
 *
 * *****************  Version 8  *****************
 * User: Jari         Date: 10/22/97   Time: 6:41p
 * Updated in $/Products/Bld8-20/Design/Drv/W95/VxD
 * Added pack instruction macros for VC++ 5.0
 *
 * *****************  Version 7  *****************
 * User: Jari         Date: 10/21/97   Time: 3:00p
 * Updated in $/Products/Bld8-20/Design/Drv/W95/VxD
 * Added an RC code
 *
 * *****************  Version 6  *****************
 * User: Jari         Date: 10/21/97   Time: 1:38p
 * Updated in $/Products/Bld8-20/Design/Drv/W95/VxD
 * Added RC code
 *
 * *****************  Version 5  *****************
 * User: Jari         Date: 10/20/97   Time: 12:05p
 * Updated in $/Products/Bld8-20/Design/Drv/W95/VxD
 * Moved BOARD_PER_PC into blddef.h from Core
 *
 * *****************  Version 4  *****************
 * User: Jari         Date: 10/18/97   Time: 3:29p
 * Updated in $/Products/Bld8-20/Design/Drv/W95/VxD
 *
 * *****************  Version 3  *****************
 * User: Jari         Date: 10/17/97   Time: 2:39p
 * Updated in $/Products/Bld8-20/Design/Drv/W95/VxD
 *
 * *****************  Version 2  *****************
 * User: Hanz         Date: 10/16/97   Time: 3:50p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc
 * Added one more error code.
 *
 * *****************  Version 1  *****************
 * User: Jari         Date: 10/14/97   Time: 5:54p
 * Created in $/Products/Bld8-20/Design/Drv/Dos/Bc
 * Balder-8 API type and macro definitions
 *
 */

#endif   // BLDDEF_H
